import { Switch, Route, Redirect } from "wouter";
import { queryClient } from "./lib/queryClient";
import { QueryClientProvider } from "@tanstack/react-query";
import { Toaster } from "@/components/ui/toaster";
import { TooltipProvider } from "./components/ui/tooltip";
import { configureToastDuration } from "@/hooks/use-toast";
import { AuthProvider, useAuth } from "@/hooks/use-auth";
import { WaitlistProvider } from "@/hooks/use-waitlist";
import { ProfileProvider } from "@/hooks/use-profile-context";
import { ErrorBoundary } from "@/components/error-boundary";
// Temporarily removed feedback button import
// import { FeedbackButton } from "./components/feedback-button";
import { ProtectedRoute } from "./lib/protected-route";
import { WaitlistCheck } from "@/components/waitlist-check";
import NotFound from "@/pages/not-found";
import AuthPage from "@/pages/auth-page";
import HomePage from "@/pages/home-page";
import TemplatesPage from "@/pages/templates-page";
import ReviewGeneratorPage from "@/pages/review-generator-page";
import ReviewsListPage from "@/pages/reviews-list-page";
import ReviewDetailPage from "@/pages/review-detail-page";
import FeatureFlagsPage from "@/pages/admin/feature-flags";
import ResetPasswordPage from "@/pages/reset-password-page";
import ProfileSetupPage from "@/pages/profile-setup-page";
import VerifyEmailPage from "@/pages/verify-email-page";
import EmailVerificationSuccess from "@/pages/email-verification-success";
import EmailVerificationError from "@/pages/email-verification-error";
import SubscriptionPage from "@/pages/subscription-page";
import LandingPage from "@/pages/landing-page";
import LandingPageV2 from "@/pages/landing-page-v2";
import MetricsDashboardPage from "@/pages/metrics-dashboard-page";
import PerformanceInsightsPage from "@/pages/performance-insights-page";
import AudioTranscriptionsPage from "@/pages/audio-transcriptions-page";
import AudioTranscriptionsListPage from "@/pages/audio-transcriptions-list";
import AudioTranscriptionDetailPage from "@/pages/audio-transcription-detail";
import AudioTranscriptionViewerPage from "@/pages/audio-transcription-viewer";
import flagsmith from "flagsmith";
import { FlagsmithProvider } from "flagsmith/react";
import { useFlags } from "flagsmith/react";
import { HelmetProvider } from "react-helmet-async";
import React from 'react';
import SettingsPage from "@/pages/settings-page";
import PrivacyPolicyPage from "@/pages/privacy-policy-page";
import TermsOfServicePage from "@/pages/terms-of-service-page";
import ProjectsPage from "@/pages/projects-page";
import ProjectDetailPage from "@/pages/project-detail-page";
import ComponentTestPage from "@/pages/component-test-page";
import ToastDemoPage from "@/pages/toast-demo-page";
import FeedbackPage from "@/pages/feedback";
import FeedbackTemplateCreatePage from "@/pages/feedback-template-create";

import MeetingCompanionLive from "@/pages/meeting-companion-live";
import FeedbackRequestCreatePage from "@/pages/feedback-request-create";
import FeedbackSubmitPage from "@/pages/feedback-submit";

// Feature Flag aware route component that only renders children if flag is enabled
type FeatureFlagRouteProps = {
  flagName: string;
  children: React.ReactNode;
  fallback?: React.ReactNode;
};

// Export the component so it can be used in the Router
export function FeatureFlagRoute({ flagName, children, fallback = null }: FeatureFlagRouteProps) {
  const flags = useFlags([flagName]);
  // Flagsmith flags can be boolean values directly
  const isEnabled = typeof flags[flagName] === 'boolean'
    ? flags[flagName]
    : (flags[flagName] as any)?.enabled;

  if (!isEnabled) {
    return <>{fallback}</>;
  }

  return <>{children}</>;
}

// Note: We'll initialize external logging after auth is ready
import { initializeExternalLogging } from "./lib/external-logging";

// Adapter component that connects AuthProvider to ProfileProvider
function AuthProviderToProfileAdapter() {
  const { user, registerLogoutCallback, unregisterLogoutCallback } = useAuth();

  return (
    <ProfileProvider
      user={user}
      registerLogoutCallback={registerLogoutCallback}
      unregisterLogoutCallback={unregisterLogoutCallback}
    >
      <WaitlistProvider>
        <ExternalLoggingInitializer>
          <TooltipProvider>
            <Router />
            <Toaster />
          </TooltipProvider>
        </ExternalLoggingInitializer>
      </WaitlistProvider>
    </ProfileProvider>
  );
}

// Wrapper component to initialize external logging with user context
function ExternalLoggingInitializer({ children }: { children: React.ReactNode }) {
  const { user } = useAuth();

  React.useEffect(() => {
    // Initialize external logging with user ID if available
    initializeExternalLogging(user?.id);
  }, [user?.id]);

  return <>{children}</>;
}

interface WithProtectedAppProps {
  children?: React.ReactNode;
}

const withProtectedApp = (WrappedComponent: React.ComponentType<WithProtectedAppProps>) => {
  return function WithProtectedAppWrapper(props: WithProtectedAppProps) {
    const { user } = useAuth();

    // If the user's email is not verified, we don't even reach the waitlist check
    // The ProtectedRoute component will handle the redirect to the verification page

    return (
      <WaitlistCheck>
        <WrappedComponent {...props} />
      </WaitlistCheck>
    );
  };
};

// Root route handler - show landing page for anonymous users, redirect logged in users to app
function RootRouteHandler() {
  const { user } = useAuth();
  const flags = useFlags(['landing_page_v2']);
  const useLandingPageV2 = flags.landing_page_v2;

  if (user) {
    return <Redirect to="/app" />;
  }

  return useLandingPageV2 ? <LandingPageV2 /> : <LandingPage />;
}

const ProtectedHomePage = withProtectedApp(HomePage);
const ProtectedTemplatesPage = withProtectedApp(TemplatesPage);
const ProtectedReviewGeneratorPage = withProtectedApp(ReviewGeneratorPage);
const ProtectedReviewsListPage = withProtectedApp(ReviewsListPage);
const ProtectedReviewDetailPage = withProtectedApp(ReviewDetailPage);
const ProtectedSubscriptionPage = withProtectedApp(SubscriptionPage);
const ProtectedProjectsPage = withProtectedApp(ProjectsPage);
const ProtectedProjectDetailPage = withProtectedApp(ProjectDetailPage);
const ProtectedMetricsDashboardPage = withProtectedApp(MetricsDashboardPage);
const ProtectedPerformanceInsightsPage = withProtectedApp(PerformanceInsightsPage);
const ProtectedAudioTranscriptionsListPage = withProtectedApp(AudioTranscriptionsListPage);
const ProtectedAudioTranscriptionDetailPage = withProtectedApp(AudioTranscriptionDetailPage);
const ProtectedAudioTranscriptionViewerPage = withProtectedApp(AudioTranscriptionViewerPage);
const ProtectedFeedbackPage = withProtectedApp(FeedbackPage);
const ProtectedFeedbackTemplateCreatePage = withProtectedApp(FeedbackTemplateCreatePage);
const ProtectedFeedbackRequestCreatePage = withProtectedApp(FeedbackRequestCreatePage);

const ProtectedMeetingCompanionLive = withProtectedApp(MeetingCompanionLive);

function Router() {
  return (
    <Switch>
      {/* Public routes - accessible without authentication */}
      <Route path="/auth" component={AuthPage} />
      <Route path="/reset-password/:token" component={ResetPasswordPage} />
      <Route path="/email-verification-success" component={EmailVerificationSuccess} />
      <Route path="/email-verification-error" component={EmailVerificationError} />
      <Route path="/privacy-policy" component={PrivacyPolicyPage} />
      <Route path="/terms-of-service" component={TermsOfServicePage} />

      {/* Public feedback submission - accessible without authentication but requires login to submit */}
      <Route path="/feedback/submit/:token" component={FeedbackSubmitPage} />

      {/* Landing page is shown for unauthenticated users at root path */}
      <Route path="/" component={RootRouteHandler} />

      <ProtectedRoute path="/auth/verify-email">
        <VerifyEmailPage />
      </ProtectedRoute>

      {/* Profile setup is protected but not behind waitlist */}
      <ProtectedRoute path="/profile/setup">
        <ProfileSetupPage />
      </ProtectedRoute>

      {/* Protected routes behind waitlist */}
      <ProtectedRoute path="/app">
        <ProtectedHomePage />
      </ProtectedRoute>
      <ProtectedRoute path="/metrics">
        <ProtectedMetricsDashboardPage />
      </ProtectedRoute>
      <ProtectedRoute path="/insights">
        <ProtectedPerformanceInsightsPage />
      </ProtectedRoute>
      <ProtectedRoute path="/dashboard">
        <Redirect to="/app" />
      </ProtectedRoute>
      <ProtectedRoute path="/templates">
        <ProtectedTemplatesPage />
      </ProtectedRoute>

      {/* Review routes with A/B testing */}
      <ProtectedRoute path="/reviews/:id">
        <FeatureFlagRoute
          flagName="new_review_ui"
          fallback={<Redirect to="/reviews" />}
        >
          <ProtectedReviewDetailPage />
        </FeatureFlagRoute>
      </ProtectedRoute>

      <ProtectedRoute path="/reviews">
        <FeatureFlagRoute
          flagName="new_review_ui"
          fallback={<ProtectedReviewGeneratorPage />}
        >
          <ProtectedReviewsListPage />
        </FeatureFlagRoute>
      </ProtectedRoute>

      <ProtectedRoute path="/subscription">
        <ProtectedSubscriptionPage />
      </ProtectedRoute>
      <ProtectedRoute path="/settings">
        <SettingsPage />
      </ProtectedRoute>

      {/* Project detail page - must be placed first for correct route matching */}
      <ProtectedRoute path="/projects/:id">
        <FeatureFlagRoute
          flagName="support_project_grouping"
          fallback={<Redirect to="/app" />}
        >
          <ProtectedProjectDetailPage />
        </FeatureFlagRoute>
      </ProtectedRoute>

      {/* Projects main page - must be placed after detail page */}
      <ProtectedRoute path="/projects">
        <FeatureFlagRoute
          flagName="support_project_grouping"
          fallback={<Redirect to="/app" />}
        >
          <ProtectedProjectsPage />
        </FeatureFlagRoute>
      </ProtectedRoute>

      {/* Audio transcription detail page - must come before the list route */}
      <ProtectedRoute path="/audio-transcriptions/:id">
        <FeatureFlagRoute
          flagName="transcript_chat_v2"
          fallback={<ProtectedAudioTranscriptionDetailPage />}
        >
          <ProtectedAudioTranscriptionViewerPage />
        </FeatureFlagRoute>
      </ProtectedRoute>

      {/* Audio transcriptions list page */}
      <ProtectedRoute path="/audio-transcriptions">
        <ProtectedAudioTranscriptionsListPage />
      </ProtectedRoute>

      {/* Feedback system routes */}
      <ProtectedRoute path="/feedback/templates/create">
        <ProtectedFeedbackTemplateCreatePage />
      </ProtectedRoute>
      <ProtectedRoute path="/feedback/requests/create">
        <ProtectedFeedbackRequestCreatePage />
      </ProtectedRoute>
      <ProtectedRoute path="/feedback">
        <ProtectedFeedbackPage />
      </ProtectedRoute>

      <ProtectedRoute path="/admin/feature-flags">
        <FeatureFlagsPage />
      </ProtectedRoute>

      {/* Component test page for showcasing reusable components */}
      <Route path="/component-test" component={ComponentTestPage} />

      {/* Toast demo page for testing toast functionality */}
      <Route path="/toast-demo" component={ToastDemoPage} />



      <ProtectedRoute path="/meeting-companion">
        <ProtectedMeetingCompanionLive />
      </ProtectedRoute>

      <Route component={NotFound} />
    </Switch>
  );
}

function App() {
  // Set default toast duration to 5 seconds (5000ms)
  React.useEffect(() => {
    configureToastDuration(5000);
  }, []);

  return (
    <ErrorBoundary>
      <QueryClientProvider client={queryClient}>
        <HelmetProvider>
          <FlagsmithProvider
            options={{
              environmentID: import.meta.env.VITE_FLAGSMITH_ENV_ID,
              api: import.meta.env.VITE_FLAGSMITH_API_HOST,
            }}
            flagsmith={flagsmith}
          >
            <AuthProvider>
              <AuthProviderToProfileAdapter />
            </AuthProvider>
          </FlagsmithProvider>
        </HelmetProvider>
      </QueryClientProvider>
    </ErrorBoundary>
  );
}

export default App;
